@@ -7,8 +7,6 @@ from turing.generated.model_utils import OpenApiModel
 
 @dataclass
 class ResourceRequest:
-    min_allowed_replica: ClassVar[int] = 0
-    max_allowed_replica: ClassVar[int] = 20
     min_replica: int
     max_replica: int
     cpu_request: str
@@ -25,7 +23,7 @@ class ResourceRequest:
 
     @min_replica.setter
     def min_replica(self, min_replica: int):
-        if hasattr(self, 'max_replica'):
+        if hasattr(self, "max_replica"):
             ResourceRequest._verify_min_max_replica(min_replica, self.max_replica)
         self._min_replica = min_replica
 
@@ -35,7 +33,7 @@ class ResourceRequest:
 
     @max_replica.setter
     def max_replica(self, max_replica: int):
-        if hasattr(self, 'min_replica'):
+        if hasattr(self, "min_replica"):
             ResourceRequest._verify_min_max_replica(self.min_replica, max_replica)
         self._max_replica = max_replica
 
@@ -57,15 +55,7 @@ class ResourceRequest:
 
     @classmethod
     def _verify_min_max_replica(cls, min_replica: int, max_replica: int):
-        if min_replica < ResourceRequest.min_allowed_replica:
-            raise InvalidReplicaCountException(
-                f"Min replica count must be >= {ResourceRequest.min_allowed_replica}; "
-                f"min_replica passed: {min_replica}")
-        elif max_replica > ResourceRequest.max_allowed_replica:
-            raise InvalidReplicaCountException(
-                f"Max replica count must be <= {ResourceRequest.max_allowed_replica}; "
-                f"min_replica passed: {max_replica}")
-        elif min_replica > max_replica:
+        if min_replica > max_replica:
             raise InvalidReplicaCountException(
                 f"Min replica must be <= max_replica; "
                 f"min_replica passed: {min_replica}, max_replica passed: {max_replica}"
@@ -76,7 +66,7 @@ class ResourceRequest:
             min_replica=self.min_replica,
             max_replica=self.max_replica,
             cpu_request=self.cpu_request,
-            memory_request=self.memory_request
+            memory_request=self.memory_request,
         )
 
 
